@@ -3,15 +3,15 @@ from functools import wraps as functools_wraps
 import inspect
 from enum import Enum
 from dataclasses import is_dataclass
-from typing import Any
-import warnings
+from typing import Any, Callable
 from typing_extensions import Annotated, _AnnotatedAlias
 
-from magicclass.fields import MagicField, MISSING
-
-from .class_gui import (
-    ClassGui, 
+from .gui.class_gui import (
+    ClassGuiBase, 
+    ClassGui,
+    GroupBoxClassGui,
     MainWindowClassGui,
+    SubWindowsClassGui,
     ScrollableClassGui,
     ButtonClassGui, 
     CollapsibleClassGui,
@@ -21,10 +21,11 @@ from .class_gui import (
     ToolBoxClassGui,
     ListClassGui,
     )
-from ._base import PopUpMode, ErrorMode, defaults
-from .menu_gui import ContextMenuGui, MenuGui, MenuGuiBase
+from .gui._base import PopUpMode, ErrorMode, defaults, MagicTemplate
+from .gui import ContextMenuGui, MenuGui, MenuGuiBase
 from .macro import Expr
-from .utils import check_collision, get_app
+from .utils import iter_members
+from ._app import get_app
 
 _BASE_CLASS_SUFFIX = "_Base"
 
@@ -38,6 +39,8 @@ class WidgetType(Enum):
     tabbed = "tabbed"
     stacked = "stacked"
     list = "list"
+    subwindows = "subwindows"
+    groupbox = "groupbox"
     mainwindow = "mainwindow"
 
 _TYPE_MAP = {
@@ -50,12 +53,26 @@ _TYPE_MAP = {
     WidgetType.tabbed: TabbedClassGui,
     WidgetType.stacked: StackedClassGui,
     WidgetType.list: ListClassGui,
+    WidgetType.groupbox: GroupBoxClassGui,
+    WidgetType.subwindows: SubWindowsClassGui,
     WidgetType.mainwindow: MainWindowClassGui,
 }
 
 def Bound(value: Any) -> _AnnotatedAlias:
     """
-    Make Annotated type from a MagicField or a method.
+    Make Annotated type from a MagicField or a method, such as:
+    
+    .. code-block:: python
+        
+        from magicclass import magicclass, field
+        
+        @magicclass
+        class MyClass:
+            i = field(int)
+            def func(self, v: Bound(i)):
+                ...
+    
+    ``Bound(value)`` is identical to ``Annotated[Any, {"bind": value}]``.    
     """    
     # It is better to annotate like Annotated[int, {...}] but some widgets does not
     # support bind. Also, we must ensure that parameters annotated with "bind" creates
@@ -69,20 +86,21 @@ def magicclass(class_: type|None = None,
                labels: bool = True, 
                name: str = None,
                close_on_run: bool = None,
-               popup: bool = True,
                popup_mode: str | PopUpMode = None,
                error_mode: str | ErrorMode = None,
                widget_type: str | WidgetType = WidgetType.none,
                parent = None
-               ) -> ClassGui:
+               ) -> ClassGui | Callable[[type], ClassGui]:
     """
     Decorator that can convert a Python class into a widget.
     
-    >>> @magicclass
-    >>> class C:
-    >>>     ...
-    >>> c = C(a=0)
-    >>> c.show()
+    .. code-block:: python
+    
+        @magicclass
+        class C:
+            ...
+        c = C(a=0)
+        c.show()
             
     Parameters
     ----------
@@ -121,8 +139,6 @@ def magicclass(class_: type|None = None,
         
     widget_type = WidgetType(widget_type)
     
-    popup_mode = _popup_deprecation(popup, popup_mode)
-    
     def wrapper(cls) -> ClassGui:
         if not isinstance(cls, type):
             raise TypeError(f"magicclass can only wrap classes, not {type(cls)}")
@@ -131,7 +147,9 @@ def magicclass(class_: type|None = None,
         
         class_gui = _TYPE_MAP[widget_type]
         
-        check_collision(cls, class_gui)
+        if not issubclass(cls, MagicTemplate):
+            _check_collision(cls, class_gui)
+            
         # get class attributes first
         doc = cls.__doc__
         sig = inspect.signature(cls)
@@ -144,15 +162,15 @@ def magicclass(class_: type|None = None,
         
         newclass.__signature__ = sig
         newclass.__doc__ = doc
+        newclass.__module__ = mod
+        newclass.__qualname__ = qualname
         
         # concatenate annotations
         newclass.__annotations__ = class_gui.__annotations__.copy()
         newclass.__annotations__.update(annot)
-        newclass.__module__ = mod
-        newclass.__qualname__ = qualname
         
         @functools_wraps(oldclass.__init__)
-        def __init__(self, *args, **kwargs):
+        def __init__(self: MagicTemplate, *args, **kwargs):
             app = get_app() # Without "app = " Jupyter freezes after closing the window!
             macro_init = Expr.parse_init(self, cls, args, kwargs)
             class_gui.__init__(self, 
@@ -188,11 +206,11 @@ def magicclass(class_: type|None = None,
 def magicmenu(class_: type = None, 
               *, 
               close_on_run: bool = None,
-              popup: bool = True,
               popup_mode: str | PopUpMode = None,
               error_mode: str | ErrorMode = None,
+              labels: bool = True, 
               parent = None
-              ):
+              ) -> MenuGui | Callable[[type], MenuGui]:
     """
     Decorator that can convert a Python class into a menu bar.
     """    
@@ -201,22 +219,66 @@ def magicmenu(class_: type = None,
 def magiccontext(class_: type = None, 
                  *, 
                  close_on_run: bool = None,
-                 popup: bool = True,
                  popup_mode: str | PopUpMode = None,
                  error_mode: str | ErrorMode = None,
+                 labels: bool = True, 
                  parent=None
-                 ):
+                 ) -> ContextMenuGui | Callable[[type], ContextMenuGui]:
     """
     Decorator that can convert a Python class into a context menu.
     """    
     return _call_magicmenu(**locals(), menugui_class=ContextMenuGui)
 
+class MagicClassFactory:
+    """
+    Factory class that can make any magic-class.
+    """    
+    def __init__(self, 
+                 name: str,
+                 layout: str = "vertical", 
+                 labels: bool = True, 
+                 close_on_run: bool = None,
+                 popup_mode: str | PopUpMode = None,
+                 error_mode: str | ErrorMode = None,
+                 widget_type: str | WidgetType = WidgetType.none,
+                 parent = None,
+                 attrs: dict[str] = None
+                 ):
+        self.name = name
+        self.layout = layout
+        self.labels = labels
+        self.close_on_run = close_on_run
+        self.popup_mode = popup_mode
+        self.error_mode = error_mode
+        self.widget_type = widget_type
+        self.parent = parent
+        self.attrs = attrs
+    
+    def as_magicclass(self) -> ClassGuiBase:
+        _cls = type(self.name, (), self.attrs)
+        cls = magicclass(_cls, layout=self.layout, labels=self.labels, close_on_run=self.close_on_run, 
+                         name=self.name, popup_mode=self.popup_mode, error_mode=self.error_mode,
+                         widget_type=self.widget_type, parent=self.parent)
+        return cls
+    
+    def as_magicmenu(self) -> MenuGui:
+        _cls = type(self.name, (), self.attrs)
+        cls = magicmenu(_cls, close_on_run=self.close_on_run, popup_mode=self.popup_mode, 
+                        error_mode=self.error_mode, labels=self.labels, parent=self.parent)
+        return cls
+    
+    def as_magiccontext(self) -> ContextMenuGui:
+        _cls = type(self.name, (), self.attrs)
+        cls = magiccontext(_cls, close_on_run=self.close_on_run, popup_mode=self.popup_mode, 
+                           error_mode=self.error_mode, labels=self.labels, parent=self.parent)
+        return cls
+
 
 def _call_magicmenu(class_: type = None, 
                     close_on_run: bool = True,
-                    popup: bool = True,
                     popup_mode: str | PopUpMode = None,
                     error_mode: str | ErrorMode = None,
+                    labels: bool = True, 
                     parent = None,
                     menugui_class: type[MenuGuiBase] = None,
                     ):
@@ -246,8 +308,6 @@ def _call_magicmenu(class_: type = None,
     if error_mode is None:
         error_mode = defaults["error_mode"]
         
-    popup_mode = _popup_deprecation(popup, popup_mode)
-        
     if popup_mode in (PopUpMode.above, PopUpMode.below, PopUpMode.first, PopUpMode.last):
         raise ValueError(f"Mode {popup_mode.value} is not compatible with Menu.")
     
@@ -257,19 +317,25 @@ def _call_magicmenu(class_: type = None,
         elif is_dataclass(cls):
             raise TypeError("dataclass is not compatible with magicclass.")
         
-        check_collision(cls, menugui_class)
+        if not issubclass(cls, MagicTemplate):
+            _check_collision(cls, menugui_class)
+        
         # get class attributes first
         doc = cls.__doc__
         sig = inspect.signature(cls)
+        mod = cls.__module__
+        qualname = cls.__qualname__
         
         oldclass = type(cls.__name__ + _BASE_CLASS_SUFFIX, (cls,), {})
         newclass = type(cls.__name__, (menugui_class, oldclass), {})
         
         newclass.__signature__ = sig
         newclass.__doc__ = doc
+        newclass.__module__ = mod
+        newclass.__qualname__ = qualname
                 
         @functools_wraps(oldclass.__init__)
-        def __init__(self, *args, **kwargs):
+        def __init__(self: MagicTemplate, *args, **kwargs):
             app = get_app() # Without "app = " Jupyter freezes after closing the window!
             macro_init = Expr.parse_init(self, cls, args, kwargs)
             menugui_class.__init__(self, 
@@ -277,6 +343,7 @@ def _call_magicmenu(class_: type = None,
                                    close_on_run=close_on_run,
                                    popup_mode=PopUpMode(popup_mode),
                                    error_mode=ErrorMode(error_mode),
+                                   labels=labels, 
                                    )
             super(oldclass, self).__init__(*args, **kwargs)
             self._convert_attributes_into_widgets()
@@ -299,14 +366,16 @@ def _call_magicmenu(class_: type = None,
 magicmenu.__doc__ += _call_magicmenu.__doc__
 magiccontext.__doc__ += _call_magicmenu.__doc__
 
-def _popup_deprecation(popup, popup_mode):
-    if not popup:
-        msg = "'popup' is deprecated. You can select popup mode from 'first', 'last', 'dock', 'above', " \
-              "'below' or 'parentlast', and call this function like >>> @magicclass(popup_mode='first')"
-        warnings.warn(msg, DeprecationWarning)
-        return PopUpMode.parentlast
-    else:
-        return popup_mode
+
+def _check_collision(cls0: type, cls1: type):
+    """
+    Check if two classes have name collisions.
+    """    
+    mem0 = set(x[0] for x in iter_members(cls0))
+    mem1 = set(x[0] for x in iter_members(cls1))
+    collision = mem0 & mem1
+    if collision:
+        raise AttributeError(f"Collision between {cls0.__name__} and {cls1.__name__}: {collision}")
 
 class _CallableClass:
     def __init__(self):
@@ -339,18 +408,26 @@ class Parameters(_CallableClass):
         
         self.__signature__ = inspect.Signature(sig)
         
-    def __call__(self, *args):
+    def __call__(self, *args) -> None:
         params = list(self.__signature__.parameters.keys())[1:]
         for a, param in zip(args, params):
             setattr(self, param, a)
 
-class Info(_CallableClass):
-    def __call__(self):
-        from .widgets import ConsoleTextEdit
-        from .utils import extract_tooltip
-        win = ConsoleTextEdit(name="macro")
-        win.read_only = True
-        win.value = extract_tooltip(self.__class__)
-        win.show()
-        return None
+    def as_dict(self) -> dict[str, Any]:
+        """
+        Convert parameter fields into a dictionary.
+        
+        .. code-block:: python
+        
+            class params(Parameters):
+                i = 1
+                j = 2
+            
+            p = params()
+            p.as_dict() # {"i": 1, "j": 2}
+            
+        """        
+        params = list(self.__signature__.parameters.keys())[1:]
+        return {param: getattr(self, param) for param in params}
+    
     
\ No newline at end of file
